package service

import (
	"erpims/application/domain/advertising"
	"erpims/oss"
	"git.sxidc.com/go-framework/baize/convenient/entity_crud"
	"git.sxidc.com/go-framework/baize/framework/binding"
	"git.sxidc.com/go-framework/baize/framework/core/api"
	"git.sxidc.com/go-framework/baize/framework/core/api/request"
	"git.sxidc.com/go-framework/baize/framework/core/api/response"
	"git.sxidc.com/go-framework/baize/framework/core/application"
	"git.sxidc.com/go-framework/baize/framework/core/domain"
	"git.sxidc.com/go-framework/baize/framework/core/domain/entity"
	"git.sxidc.com/go-framework/baize/framework/core/infrastructure"
	"git.sxidc.com/go-framework/baize/framework/core/infrastructure/database"
	"git.sxidc.com/go-framework/baize/framework/core/infrastructure/database/sql"
	"git.sxidc.com/go-tools/utils/strutils"
	"github.com/pkg/errors"
)

var advertisingService = &AdvertisingService{}

type AdvertisingService struct{}

func (svc *AdvertisingService) Init(appInstance *application.App) error {
	svc.v1(appInstance)
	return nil
}

func (svc *AdvertisingService) Destroy() error {
	return nil
}

func (svc *AdvertisingService) v1(appInstance *application.App) {
	v1Binder := binding.NewBinder(appInstance.ChooseRouter(api.RouterPrefix, "v1"),
		appInstance.Infrastructure())

	entity_crud.BindSimple[advertising.Info](v1Binder,
		&entity_crud.Simple[advertising.Info]{
			Entity:             &advertising.Entity{},
			Schema:             dbSchema,
			CreateJsonBody:     &advertising.CreateAdvertisingJsonBody{},
			DeleteQueryParams:  &advertising.DeleteAdvertisingQueryParams{},
			UpdateJsonBody:     &advertising.UpdateAdvertisingJsonBody{},
			QueryQueryParams:   &advertising.GetAdvertisingsQueryParams{},
			GetByIDQueryParams: &advertising.GetAdvertisingQueryParams{},
		},
		entity_crud.WithCreateTx(),
		entity_crud.WithCreateCallbacks(&entity_crud.CreateCallbacks{
			Before: func(c *api.Context, params request.Params, e entity.Entity, prepared map[string]any, i *infrastructure.Infrastructure, tx database.Executor) error {
				requestParams, err := request.ToConcrete[*advertising.CreateAdvertisingJsonBody](params)
				if err != nil {
					return err
				}
				info, err := domain.ToConcrete[*advertising.Entity](e)
				if err != nil {
					return err
				}

				//图片处理
				if strutils.IsStringNotEmpty(requestParams.PicUrl) {
					newPicture, err := oss.MultipleFileMv(requestParams.PicUrl)
					if err != nil {
						return err
					}
					info.PicUrl = newPicture
				}

				return nil
			},
		}),
		entity_crud.WithUpdateTx(),
		entity_crud.WithUpdateCallbacks(&entity_crud.UpdateCallbacks{
			Before: func(c *api.Context, params request.Params, e entity.Entity, prepared map[string]any, i *infrastructure.Infrastructure, tx database.Executor) error {
				requestParams, err := request.ToConcrete[*advertising.UpdateAdvertisingJsonBody](params)
				info, err := domain.ToConcrete[*advertising.Entity](e)
				if err != nil {
					return err
				}

				if strutils.IsStringNotEmpty(requestParams.PicUrl) {
					newPicture, err := oss.MultipleFileMv(requestParams.PicUrl)
					if err != nil {
						return err
					}
					info.PicUrl = newPicture
				}
				return nil
			},
		}),
		entity_crud.WithQueryCallbacks(&entity_crud.QueryCallbacks[advertising.Info]{
			OnSuccessReturn: func(c *api.Context, params request.Params, e entity.Entity, i *infrastructure.Infrastructure, output response.InfosData[advertising.Info]) (response.InfosData[advertising.Info], error) {
				for i2 := range output.Infos {
					item := &output.Infos[i2]
					if strutils.IsStringNotEmpty(item.PicUrlStr) {
						fileObjects, err := oss.MultipleFileUrl(item.PicUrlStr)
						if err != nil {
							return output, err
						}
						item.PicUrl = fileObjects
					}
				}
				return output, nil
			},
		}),
		entity_crud.WithGetByIDCallbacks(&entity_crud.GetByIDCallbacks[advertising.Info]{
			OnSuccessReturn: func(c *api.Context, params request.Params, e entity.Entity, i *infrastructure.Infrastructure, output advertising.Info) (advertising.Info, error) {
				if strutils.IsStringNotEmpty(output.PicUrlStr) {
					fileObjects, err := oss.MultipleFileUrl(output.PicUrlStr)
					if err != nil {
						return output, err
					}
					output.PicUrl = fileObjects
				}

				return output, nil
			},
		}),
		entity_crud.WithQueryOrderBy[advertising.Info](entity.ColumnCreatedTime+" desc"),
		entity_crud.WithQueryConditionFieldCallback[advertising.Info](func(conditions *sql.Conditions, fieldName string, columnName string, value any) (hasDeal bool, err error) {
			switch fieldName {
			case advertising.FieldTitle:
				conditions.Like(columnName, "%"+value.(string)+"%")
				return true, nil
			case advertising.ColumnPageUrl:
				conditions.Like(columnName, "%"+value.(string)+"%")
				return true, nil
			default:
				return false, nil
			}
		}))
}

func (svc *AdvertisingService) queryByKeyFields(e *advertising.Entity, dbExecutor database.Executor) (*advertising.Entity, error) {
	result, err := database.QueryOne(dbExecutor, &sql.QueryOneExecuteParams{
		TableName:  domain.TableName(dbSchema, e),
		Conditions: sql.NewConditions().Equal(entity.ColumnID, e.ID),
	})
	if err != nil {
		if database.IsErrorDBRecordNotExist(err) {
			return nil, errors.New(e.DomainCNName() + "不存在")
		}

		return nil, err
	}

	existEntity := new(advertising.Entity)
	err = sql.ParseSqlResult(result, existEntity)
	if err != nil {
		return nil, err
	}

	return existEntity, nil
}

func (svc *AdvertisingService) checkExistByKeyFields(e *advertising.Entity, dbExecutor database.Executor) error {
	exist, err := database.CheckExist(dbExecutor, &sql.CheckExistExecuteParams{
		TableName:  domain.TableName(dbSchema, e),
		Conditions: sql.NewConditions().Equal(entity.ColumnID, e.ID),
	})
	if err != nil {
		return err
	}

	if !exist {
		return errors.New(e.DomainCNName() + "不存在")
	}

	return nil
}
